#! /bin/csh -f
#
#	general script for converting mtz to another format
#
##############################################################################
# set up awk
alias nawk /bin/gawk
nawk 'BEGIN{print 1; exit}' >& /dev/null
if($status) alias nawk awk

# defaults
set mtzfile  = merged.mtz
set tempfile = mtz2various_temp

# output F, phase, etc. (automatic by default)
set hiRES    = ""
set F        = Fin
set SIGF     = SIGFin
set DANO     = ""
set SIGD     = ""
set PHI      = PHIDM
set FOM      = FOMDM
set FREE     = FreeR_flag

# output files
set outfile   = outfile.cif
# supported: xplor cns tnt cif shelx fin phs
set format    = "CIF"

if("$1" == "") goto Help
goto Setup
#  Procedure (at bottom) to read command-line args
#  mtz, Fs, or resolution
Help:
cat << EOF

usage: $0 mtzfile.mtz [F] [PHI] outfile.fmt

where:
mtzfile.mtz  - an mtz file you want to convert
[F] [PHI]    - are the dataset names in mtzfile.mtz you want to extract
outfile.fmt  - is the output file name
fmt          - implies format:
		cif -> CIF
		hkl -> shelx
		tnt -> TNT
		fin -> XtalView
		phs -> XtalView
		fobs-> XPLOR
		cv  -> XPLOR
		cns -> CNS

EOF
exit 9
ReturnFromSetup:

# construct a LABIN card for the dump
set label = ""
if("$F" != "")    set label = "$label FP=$F"
if("$SIGF" != "") set label = "$label SIGFP=$SIGF"
if("$DANO" != "") set label = "$label DP=$DANO"
if("$SIGD" != "") set label = "$label SIGDP=$SIGD"
if("$PHI" != "")  set label = "$label PHIB=$PHI"
if("$FOM" != "")  set label = "$label FOM=$FOM"
if("$FREE" != "") set label = "$label FREE=$FREE"

################################################################################
# extract columns of interest
mtz2various HKLIN $mtzfile HKLOUT ${tempfile}.hkl << EOF-dump 
#>> /dev/null
RESOLUTION 1000 $hiRES
OUTPUT $format
LABIN $label
END
EOF-dump
if($status) exit

if("$format" == "TNT") then
    if("$FREE" != "") then
	# extract FREE-R set
	cat ${tempfile}.hkl |\
	nawk '$NF=="FREE"{print substr($0,1,index($0,"FREE")-1)}' |\
	sort +1n -2 +2n -3 +3n -4 |\
	cat >! ${outfile}.free
    endif
    # make sure the output is properly sorted
    cat ${tempfile}.hkl |\
    nawk '$NF!="FREE"{print}' |\
    sort +1n -2 +2n -3 +3n -4 |\
    cat >! ${outfile}

    echo "TNT version of $F $SIGF $PHI $FOM in $mtzfile is now in:"
    echo "$outfile"
    echo "${outfile}.free contains the Free-R flagged HKLs"
    rm -f ${tempfile}.hkl >& /dev/null
    exit
endif

# check for supported/unsupported output
if("$format" != 'USER *') then
    # use whatever mtz2various did
    mv ${tempfile}.hkl $outfile

    echo "$format version of $F $SIGF $DANO $SIGD $PHI $FOM from $mtzfile is at:"
    echo "$outfile"
    exit
endif

# must have been an unsupported format (fin, phs)
if(("$ext" == "fin")||("$ext" == "phs")) then
    echo "name" >! ${outfile}.CRYSTAL
    echo "cell $CELL" >> ${outfile}.CRYSTAL
    cat ${CLIBD}/symop.lib |\
    nawk -v SG=$SG '$4==SG{print "spgr", tolower($4), $1, $2, $3, tolower($6);\
	getline; getline; printf "symm x,y,z"; \
	while(NF==1){printf "; %s", tolower($1); getline};\
	print "."; exit}' |\
    cat >> ${outfile}.CRYSTAL
endif

if("$ext" == "phs") then
    # XtalView phs file
    # should have been output as H K L F PHI FOM
    cat ${tempfile}.hkl |\
    nawk '{printf "%4d %4d %4d %12.4f %7.4f %7.2f\n", $1, $2, $3, $4, $6, $5}' |\
    cat >! $outfile

    echo "XtalView version of $F $PHI $FOM from $mtzfile is at:"
    echo "$outfile"
    echo "${outfile}.CRYSTAL"
    rm -f ${tempfile}.hkl >& /dev/null
    exit
endif


if("$ext" == "fin") then
    # XtalView fin file
    # should have been output as H K L F SIGF D SIGD
    cat ${tempfile}.hkl |\
    nawk 'NF>=7{F1=$4+($6/2); F2=$4-($6/2); SIG1=SIG2=$5*1.4142;}\
	  NF<7 || $NF+0==0{F1=$4; SIG1=$5; F2=0; SIG2=9999}\
	  NF>3{printf "%4d %4d %4d %12.4f %12.4f %12.4f %12.4f\n", $1, $2, $3, F1, SIG1, F2, SIG2}' |\
    cat >! $outfile

    echo "XtalView version of $F $SIGF $DANO $SIGD from $mtzfile is at:"
    echo "$outfile"
    echo "${outfile}.CRYSTAL"
    rm -f ${tempfile}.hkl >& /dev/null
    exit
endif


####################################################################
exit
####################################################################



Setup:
# scan the command line for files
foreach arg ( $* )
    if( "$arg" =~ *.mtz ) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set mtzfile  = "$arg"
	continue
    endif
    if("$arg" =~ *.cif) then
        set outfile  = "$arg"
        set format   = "CIF data_$arg"
        continue
    endif
    if("$arg" =~ *.hkl) then
	set outfile  = "$arg"
	set format   = "SHELX"
	continue
    endif
    if("$arg" =~ *.tnt) then
        set outfile  = "$arg"
        set format   = "TNT"
        continue
    endif
    if("$arg" =~ *.cns) then
        set outfile  = "$arg"
        set format   = "CNS"
        continue
    endif
    if(("$arg" =~ *.fobs)||("$arg" =~ *.cv)) then
	set outfile  = "$arg"
	set format   = "XPLOR"
	continue
    endif
    if("$arg" =~ *.fin) then
        set outfile  = "$arg"
        set format   = 'FIN'
        continue
    endif
    if("$arg" =~ *.phs) then
        set outfile  = "$arg"
        set format   = 'PHS'
        continue
    endif
    if( "$arg" =~ [0-9]* ) then
	set temp = `echo "$arg" | nawk '$1+0>0.1{print $1+0}'`
	if("$temp" != "") set hiRES = "$temp"
    endif
end

#get variables from mtz file
echo "go" | mtzdump hklin $mtzfile | tee ${tempfile}mtzhead |\
nawk '/OVERALL FILE STATISTICS/,/No. of reflections used/' |\
nawk 'NF>10 && $(NF-1) ~ /[FQPWADI]/' |\
cat >! ${tempfile}mtzdmp

# set misc header values
set CELL = `nawk '/Cell Dimensions/{getline; getline; print}' ${tempfile}mtzhead`
set SG   = `nawk '/Space group/{gsub("\047","");print $5}' ${tempfile}mtzhead`
set SGnum = ` nawk '/Space group/{print $NF+0}' ${tempfile}mtzhead `
set SG = ` nawk -F "[\047]" '/Space group/{print $2}' ${tempfile}mtzhead `
set SG = ` nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib `
set mtzRES = `nawk '/Resolution Range/{getline;getline;print $6}' ${tempfile}mtzhead`
rm -f ${tempfile}mtzhead >& /dev/null

# use completeness, or F/sigF to pick default F
cat ${tempfile}mtzdmp |\
nawk '$(NF-1) == "F"{F=$NF; meanF=$8; reso=$(NF-2); comp=substr($0,32)+0; \
      getline; S=$NF; if($8) meanF /= $8; print F, S, reso, comp, meanF;}' |\
sort +2n -3 +3nr -4 +4nr >! ${tempfile}F

# same for D/sigD for anomalous diffs
cat ${tempfile}mtzdmp |\
nawk '$(NF-1) == "D"{D=$NF; meanD=$8; reso=$(NF-2); comp=substr($0,32)+0; \
      getline; S=$NF; if($8) meanD /= $8; print D, S, reso, comp, meanD;}' |\
sort +2n -3 +3nr -4 +4nr >! ${tempfile}D


# and extract all dataset types/labels
cat ${tempfile}mtzdmp |\
nawk 'NF>2{print $(NF-1), $NF, " "}' |\
cat >! ${tempfile}cards

#clean up
rm -f ${tempfile}mtzdmp

# pick F with best resolution, or <F>/<sigma(F)>
set F    = `head -1 ${tempfile}F`
if($#F > 2) then
    set SIGF = $F[2]
    set F    = $F[1]
endif

# see if default phase is available in this file
grep "P $PHI" ${tempfile}cards >& /dev/null
if($status) then
    # pick most recently-added phase
    set temp = `nawk '/^P/{print $2}' ${tempfile}cards  | tail -1`
    if("$temp" != "") set PHI = "$temp"
endif
# see if default FOM is available in this file
grep "W $FOM" ${tempfile}cards >& /dev/null
if($status) then
    # pick most recently-added FOM
    set temp = `nawk '/^W/{print $2}' ${tempfile}cards | tail -1`
    if("$temp" != "") then
	set FOM = "$temp"
    else
	# there are no FOMs in this mtz file
	set FOM = ""
    endif
endif

# see if user specified an F, Phase, or FOM
set last = "F"
foreach arg ( $* )
    set temp = `grep " $arg " ${tempfile}cards`
    if("$temp" =~ F*) then
	set F = "${arg}"
	set last = "F"
	# assign most likely sigma too
	set temp = `nawk -v arg="$arg" '$1==arg{print $2}' ${tempfile}F`
	if($#temp == 1) set SIGF = "$temp"
	continue
    endif
    if("$temp" =~ D*) then
        set DANO = "${arg}"
	set last = "D"
        # assign most likely sigma too
        set temp = `nawk -v arg="$arg" '$1==arg{print $2}' ${tempfile}D`
        if($#temp == 1) set SIGD = "$temp"
        continue
    endif
    if("$temp" =~ Q*) then
	if("$last" == "F") set SIGF = "${arg}"
	if("$last" == "D") set SIGD = "${arg}"
    endif
    if("$temp" =~ P*) set PHI  = "${arg}"
    if("$temp" =~ W*) set FOM  = "${arg}"


    # detect particular out-of-context format specifiers
    set arg = `echo "$arg" | nawk '{print tolower($1)}'`
    if("$arg" == "shelx") set format = SHELX
    if("$arg" == "tnt")   set format = TNT
    if("$arg" == "cif")   set format = CIF
    if("$arg" == "xplor") set format = XPLOR
    if("$arg" == "cns")   set format = CNS
    if("$arg" == "xtalview") then
        if("$PHI" == "") then
            set format = "FIN"
        else
            set format = "PHS"
        endif
    endif
end

# check for DANO, if we might need one
if(("$format" == "FIN")&&("$DANO" == "")) then
    # grab pattern from name
    set temp = `echo $F | nawk '{print substr($1,2)}'`
    set DANO = `nawk -v patt=$temp '$1~ patt"$" {print $1; exit}' ${tempfile}D`
    # get likely sigma too
    set SIGD = `nawk -v arg="$DANO" '$1==arg{print $2}' ${tempfile}D`
endif

# now check and see if the sigma is really there
grep "Q $SIGF" ${tempfile}cards >& /dev/null
if($status) then
    # no sigma availale
    set SIGF = ""
endif
grep "Q $SIGD" ${tempfile}cards >& /dev/null
if($status) then
    # no sigma availale 
    set SIGD = ""
endif
grep "P $PHI" ${tempfile}cards >& /dev/null
if($status) then
    # no phase availale
    set PHI = ""
endif
grep "W $FOM" ${tempfile}cards >& /dev/null
if($status) then
    # no weight availale
    set FOM = ""
endif
grep " $FREE" ${tempfile}cards >& /dev/null
if($status) then
    # no weight availale
    set FREE = ""
endif


rm -f ${tempfile}cards ${tempfile}F ${tempfile}D >& /dev/null

# set resolution (if none previously given)
if("$hiRES" == "") set hiRES = "$mtzRES"

# store filename extension/format
set ext = `echo $outfile | nawk 'BEGIN{FS="."} {print $NF}'`
if("$format" == "CIF") then
    # needs an extra word
    set format = "CIF data_dump"
endif
if("$format" == "FIN") then
    # not supported by mtz2various
    set format = 'USER *'
    set ext = fin
    set PHI = ""
    set FOM = ""
    set FREE = ""
endif
if("$format" == "PHS") then
    # not supported by mtz2various
    set format = 'USER *'
    set ext = phs
    set SIGF = ""
    set DANO = ""
    set SIGD = ""
    set FREE = ""
endif

goto ReturnFromSetup

####################
# future:
better handling of shelx
multiple datasets/mtzs?

